#include "ofApp.h"

//--------------------------------------------------------------
void ofApp::setup(){
    sampleRate = 44100;
    bufferSize = 512;
    
    gui.setup();
    // add ofParameters to "gui" panel
    gui.add(carrierFreq.set("carrier frequency", 440, 20, 1000));
    gui.add(modFreq.set("square mod frequency", 100, 0, 500));
    gui.add(modIndex.set("square mod index", 100, 0, 500));
    gui.add(gain.set("gain", 0.5, 0, 1));
    gui.add(cutoff.set("cutoff", 10, 500, 8000));
    gui.add(resonance.set("resonance", 1, 0, 100));
    gui.add(modFreq2.set("saw mod frequency", 100, 0, 500));
    gui.add(mod2Index.set("saw mod index", 100, 0, 500));
    
    // switch on the DAC - needs to happen at the end of setup
    ofSoundStreamSetup(2,2,this, sampleRate, bufferSize, 4);
    
}

//--------------------------------------------------------------
void ofApp::update(){
    
}

//--------------------------------------------------------------
void ofApp::draw(){
    ofBackground(250);
    gui.draw();
    drawWaveform();
    ofDrawBitmapString("press 1 for wobbly bass\n press 2 for guitar buzz \n press 3 for hum", 500, 50);
}

//--------------------------------------------------------------
void ofApp::audioOut(float * output, int bufferSize, int nChannels) {
    for (int i = 0; i < bufferSize; i++) {
        
        // modulators
        modOUT =  mod.square(modFreq) * modIndex;
        mod2OUT = mod2.saw(modFreq2) * mod2Index;
        // carrier
        oscOUT = carrier.sinewave(carrierFreq + modOUT + mod2OUT);
        //filter
        filterOUT = lowpass.lores(oscOUT, cutoff, resonance);
        
        mix = filterOUT * gain;
        
        output[i*nChannels    ] = mix; // left channel
        output[i*nChannels+1  ] = mix; // right channel
        
        soundBuffer[i] = mix;
    }
}

//--------------------------------------------------------------
void ofApp::drawWaveform() {
    ofNoFill();
    ofSetColor(50);
    ofSetLineWidth(5);
    ofBeginShape();
    for(int i = 0; i < bufferSize; i++) {
        float x = ofMap(i, 0, bufferSize, 0, ofGetWidth());
        if(pause == false) {
            ofVertex(x, ofGetHeight()*.5 + soundBuffer[i] * 200);
            pauseBuffer[i] = soundBuffer[i];
        } else
            ofVertex(x, ofGetHeight()*.5 + pauseBuffer[i] * 200);
    }
    ofEndShape();
}

//--------------------------------------------------------------
void ofApp::keyPressed(int key){
    if(key == 32) pause = !pause;
    //r key randomise
    if(key == 114){
        carrierFreq = ofRandom(20,1000);
        modFreq = ofRandom(0, 500);
        modIndex = ofRandom(0,500);
        gain = ofRandom(0,0.3);
        cutoff = ofRandom(0, 8000);
        resonance = ofRandom(0, 100);
        modFreq2 = ofRandom(0, 500);
        mod2Index = ofRandom(0, 500);
    }
    //1 key preset
    if(key == 49){
        carrierFreq = 417;
        modFreq = 170;
        modIndex = 293;
        gain = 0.3;
        cutoff = 2638;
        resonance = 21;
        modFreq2 = 350;
        mod2Index = 258;
    }
    //2 key preset
    if(key == 50){
        carrierFreq = 530;
        modFreq = 260;
        modIndex = 328;
        gain = 0.3;
        cutoff = 6425;
        resonance = 20;
        modFreq2 = 175;
        mod2Index = 215;
    }
    //3 key preset
    if(key == 51){
        carrierFreq = 461;
        modFreq = 220;
        modIndex = 275;
        gain = 0.3;
        cutoff = 800;
        resonance = 83;
        modFreq2 = 1;
        mod2Index = 250;
    }
    std:: cout << key << endl;
}

